from firehelper import CommandRegistry
from azkm.commands import init, deploy, dataset

def deploy_imagenet(km_id: str, region:str):
    init.deploy_km(km_id, region)
    deploy.deploy_pipeline("imagenet", km_id)
    dataset.deploy_imagenet(km_id, num_images=10)
    deploy.deploy_application("imagenet", km_id)
    
recipes = {
    'recipe': {
        'imagenet': deploy_imagenet
    }
}

CommandRegistry.register(recipes)


import shutil
from tabulate import tabulate
from azkm.utils import osutil
import os
import json

prereq_cmd = [
    'terraform',
    'kubectl'
]

cdktf_cfg = {
    "language": "python",
    "app": "python ./__main__.py",
    "terraformProviders": ["azurerm@~> 2.40.0"],
    "codeMakerOutput": "providers"
}

def check_cmd():
    prereq_paths = []
    for cmd in prereq_cmd:
        cmd_path = shutil.which(cmd)
        assert cmd_path is not None, 'Please install {0}'.format(cmd)
        prereq_paths = prereq_paths + [(cmd, cmd_path)]
    return prereq_paths

def confirm_cmd():
    prereqs = check_cmd()
    print('Pre-requisites:')
    print(tabulate(prereqs, headers=['cmd', 'path']))
    print('\r\n')

def get_providers():
    if not os.path.isdir('{0}/azkm/providers/azurerm'.format(osutil.ROOT_DIR)):
        assert shutil.which('cdktf') is not None,  'Please install cdktf: npm i cdktf-cli'
        osutil.chdir('azkm')
        with open('cdktf.json', 'w') as cfg:
            cfg.write(json.dumps(cdktf_cfg))
        osutil.run_subprocess(['cdktf', 'get'])

def install():
    check_cmd()
    get_providers()


def flatten_dict(d: dict):
    f_d = {}
    def __(d: dict, key_prefix: str = ''):
        if isinstance(d, dict):
            for k in d:
                if isinstance(d[k], dict):
                    __(d[k], key_prefix + k + '/')
                elif not isinstance(d[k], str) and isinstance(d[k], list):
                    for idx, val in enumerate(d[k]):
                        suffix = '/' + str(idx) + '/'
                        if len(d[k]) == 1:
                            suffix = '/'
                        __(val, key_prefix + k + suffix)
                else:
                    f_d[key_prefix + k] = d[k]   
        elif isinstance(d, str):  # if you got here from a list of str
            f_d[key_prefix] = d

    __(d) 
    return f_d

def replace_tokens(txt: str, ref_dict: dict):
    # flatten dict
    f_d = flatten_dict(ref_dict)
    
    txt_out = txt
    for k in f_d:
        if f_d[k]:
            txt_out = txt_out.replace('{{'+k+'}}', str(f_d[k]))
    return txt_out




import requests
from azkm.utils.stdout import stdout_print

API_VERSION = '2020-06-30'

def create_resource(component_name: str, resource_name: str, search_attr: dict, resource_attr: dict):
    stdout_print('provisioning {0} {1} in search {2}.'.format(component_name, resource_name, search_attr['name']))
    headers = {
        'Content-Type': 'application/json',
        'api-key': search_attr['primary_key']
    }
    # pluralise
    last_char = component_name[-1]
    if last_char is not 's':
        if last_char is 'x':
            component_name = component_name + 'e'
        component_name = component_name + 's'

    resource_attr['name'] = resource_name
    res_endpoint = 'https://{0}.search.windows.net/{1}?api-version={2}'.format(search_attr['name'], component_name, API_VERSION)
    check_res = requests.get('{0}&$select=name'.format(res_endpoint), headers=headers)
    check_res.raise_for_status()
    if 'value' in check_res.json() and any(v['name'] == resource_name for v in check_res.json()['value']):
        stdout_print('resource {0} with name {1} already exists in search {2}.'.format(component_name, resource_name, search_attr['name']))
        # raise Exception('{0} already exists for {1} in search appliance {2}'.format(component_name, resource_name, search_attr['name']))
    else:
        requests.post(res_endpoint, headers=headers, json=resource_attr).raise_for_status()
        stdout_print('created resource {0} with name {1} in search {2}.'.format(component_name, resource_name, search_attr['name']))
        
    
def create_datasource(resource_name: str, search_attr: dict, resource_attr: dict):
    return create_resource('datasources', resource_name, search_attr, resource_attr)

def create_index(resource_name: str, search_attr: dict, resource_attr: dict):
    return create_resource('indexes', resource_name, search_attr, resource_attr)

def create_indexer(resource_name: str, search_attr: dict, resource_attr: dict):
    return create_resource('indexers', resource_name, search_attr, resource_attr)

def create_skillset(resource_name: str, search_attr: dict, resource_attr: dict):
    return create_resource('skillsets', resource_name, search_attr, resource_attr)



"""az cli utility library."""
from azure.cli.core import get_default_cli as az


def run_command(*cmd: tuple, verbose=False):
    """Run an az cli command.

    Args:
        command (str): cli command

    Returns:
        dict: result of command
    """
    if not verbose and '--output' not in cmd:
        cmd = cmd + ('--output', 'none')
    cli = az()
    cli.invoke(cmd)
    if cli.result.error:
        raise cli.result.error
    
    return cli.result.result

def logged_in() -> bool:
    """Check if logged in to az cli.

    Returns:
        bool: logged_in
    """
    try:
        res = run_command('account', 'list')
        return len(res) > 0
    except Exception as e:
        return False

def create_sp(name: str):
    sp = run_command('ad', 'sp', 'create-for-rbac', '--name', name)
    query = "[?appId=='{0}']|[0]".format(sp['appId'])
    full_sp = run_command('ad' , 'sp', 'list', '--show-mine', '--query', query)
    sp['objectId'] = full_sp['objectId']
    return sp

def delete_sp(id: str):
    return run_command('ad', 'sp', 'delete', '--id', id)
    


"""azkm CLI entry point."""

from azkm.flight_checks import prereqs
import firehelper
import sys
from azkm.commands import *  # noqa

def main():
    """azkm CLI.
    """
    if len(sys.argv) == 1:
        prereqs.confirm_cmd()
    else:
        prereqs.check_cmd()

    start_cli()

def start_cli():
    firehelper.start_fire_cli('azkm')

if __name__ == '__main__':
    main()



import firehelper
from azkm.utils import tf

def plan_km(km_id: str, region:str):
    tf.synth_km(km_id, region)
    tf.init(km_id)
    tf.plan(km_id)

def deploy_km(km_id: str, region:str):
    tf.synth_km(km_id, region)
    tf.init(km_id)
    tf.apply(km_id)

init_cmd = {
    'init': deploy_km
}

firehelper.CommandRegistry.register(init_cmd)


"""CLI commands: fulmar deploy pipeline [ida]."""
import json
import os
import requests
from urllib.parse import urljoin

from azkm.utils import az
from azkm.utils import tf
from azkm.utils import cogsearch
from azkm.utils import text
from azkm.utils import osutil
from azkm.utils import zip
from firehelper import CommandRegistry

# TODO: rethink pipeline/application deployment config + pattern


PIPELINE_URL = 'https://raw.githubusercontent.com/frogrammer/azkm-pipelines/master/pipelines.json'
PIPELINE_COMPONENTS = ['datasource', 'skillset', 'index', 'indexer']  # installation order
APP_COMPONENTS = ['app_bin', 'app_template']
FILES_TO_TOKEN_REPLACE = ['.json', '.yaml']

def _load_pipeline(url: str, pipeline_name: str, resource_attr: dict):
    p_res = requests.get(url)
    p_res.raise_for_status()
    p_components = p_res.json()
    if not pipeline_name in p_components or not all([p in p_components[pipeline_name] for p in PIPELINE_COMPONENTS]):
        raise Exception('Invalid pipeline json.')
    
    pipeline = {}
    for p in PIPELINE_COMPONENTS:
        comp_url = p_components[pipeline_name][p]
        if comp_url.startswith('.'):  # relative url
            comp_url = urljoin(url, comp_url)
        comp_res = requests.get(comp_url)
        comp_res.raise_for_status()
        res_txt = text.replace_tokens(comp_res.text, resource_attr)
        pipeline[p] = {}
        try:
            pipeline[p] = json.loads(res_txt)
        except:
            pipeline[p] = json.loads(res_txt[1:])

    return pipeline

def _load_application(url: str, pipeline_name: str, resource_attr: dict, km_id: str):
    p_res = requests.get(url)
    p_res.raise_for_status()
    p_components = p_res.json()
    if not pipeline_name in p_components or not all([p in p_components[pipeline_name] for p in APP_COMPONENTS]):
        raise Exception('Invalid pipeline json.')
    
    pipeline_app_root = os.path.join(tf.get_out_dir(km_id), pipeline_name)

    if not os.path.isdir(pipeline_app_root):
        os.mkdir(pipeline_app_root)

    app = {}
    for p in APP_COMPONENTS:
        comp_url = p_components[pipeline_name][p]
        if comp_url.startswith('.'):  # relative url
            comp_url = urljoin(url, comp_url)
        comp_res = requests.get(comp_url)
        comp_res.raise_for_status()
        if comp_url.endswith('.zip'):
            path = os.path.join(pipeline_app_root, comp_url.split('/')[-1].replace('.zip', ''))
            zip.extract_all(comp_res.content, path)
            for root, dirs, files in os.walk(path):
                for file in files:
                    if any([file.endswith(ext) for ext in FILES_TO_TOKEN_REPLACE]):
                        file_path = os.path.join(root, file)
                        file_txt = ''
                        with open(file_path, 'r') as f:
                            file_txt = text.replace_tokens(f.read(), resource_attr)
                        with open(file_path, 'w') as f:
                            f.write(file_txt)
            app[p] = path
        else: 
            file_name = os.path.join(pipeline_app_root, comp_url.split('/')[-1])
            res_txt = comp_res.text
            if any([file_name.endswith(ext) for ext in FILES_TO_TOKEN_REPLACE]):
                res_txt = text.replace_tokens(comp_res.text, resource_attr)
            with open(file_name, 'w') as f:
                f.write(res_txt)
            app[p] = file_name

    return app

def deploy_pipeline(pipeline: str, km_id: str, pipeline_url = PIPELINE_URL):
    """Deploy pipeline.

    Args:
        km_id (str): azkm instance name
    """
    try:
        env_state = tf.get_state(km_id)
    except:
        raise Exception('Error finding search appliance or cognitive services for environment {0}'.format(km_id))    

    pipeline_resources = _load_pipeline(pipeline_url, pipeline, env_state)
    
    for p in PIPELINE_COMPONENTS:
        cogsearch.create_resource(p, pipeline, env_state['azurerm_search_service'][0], pipeline_resources[p])
        
    print('\r\nDeployed imagenet pipeline to environment {0}.'.format(km_id))


def deploy_application(pipeline: str, km_id: str, pipeline_url = PIPELINE_URL):
    """Deploy application.

    Args:
        km_id (str): azkm instance name
    """
    try:
        env_state = tf.get_state(km_id)
    except:
        raise Exception('Error finding search appliance or cognitive services for environment {0}'.format(km_id))    

    app_resources = _load_application(pipeline_url, pipeline, env_state, km_id)
    
    # copy azkm resources to file share
    az.run_command('storage', 'file', 'upload-batch', 
        '--account-name', env_state['azurerm_storage_account'][0]['name'], 
        '--account-key', env_state['azurerm_storage_account'][0]['primary_access_key'], 
        '-s', tf.get_out_dir(km_id), 
        '-d', 'kmapp')
    
    # kubectl deploy app_template
    if app_resources['app_template']:
        osutil.run_subprocess(['az', 'aks', 'get-credentials', 
        '-n', env_state['azurerm_kubernetes_cluster'][0]['name'], 
        '-g', env_state['azurerm_resource_group'][0]['name']
        ])
        osutil.run_subprocess(['kubectl', 'create', 'secret', 'generic', 'azure-secret', 
        '--from-literal=azurestorageaccountname={0}'.format(env_state['azurerm_storage_account'][0]['name']), 
        '--from-literal=azurestorageaccountkey={0}'.format(env_state['azurerm_storage_account'][0]['primary_access_key'])])
        osutil.run_subprocess(['kubectl', 'apply', '-f', app_resources['app_template']])


    print('\r\nDeployed imagenet app to environment {0}.'.format(km_id))
    #addon_profile/http_application_routing/http_application_routing_zone_name
    print('http://{0}.{1}'.format(pipeline, env_state['azurerm_kubernetes_cluster'][0]['http_application_routing'][0]['http_application_routing_zone_name']))

pipeline_commands = {
    "deploy": {
        "pipeline": deploy_pipeline,
        "app": deploy_application
    }
}

CommandRegistry.register(pipeline_commands)



class __prev(object):

    _i = 3

    @property
    def i(self):
        return type(self)._i

    @i.setter
    def i(self,val):
        type(self)._i = val


def stdout_print(text: str):
    print(' '.join(['' for i in range(0, __prev().i)]), end='\r', flush=True)
    print('{0}'.format(text), end='\r', flush=True)
    __prev().i = len(text)


from setuptools import setup, find_packages
from os import path

# https://packaging.python.org/guides/making-a-pypi-friendly-readme/
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='azkm',
    description='azure knowledge mining cli',
    version="0.2.2",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/frogrammer/azure-knowledgemining-cli',
    author='Luke Vinton',
    author_email='luke0vinton@gmail.com',
    license='Apache 2.0',
    packages=find_packages(),
    install_requires=['fire', 'fire-cli-helper', 'cdktf', 'tabulate', 'azure-cli==2.14.0', 'azure-mgmt-core==1.2.0', 'azure-storage-blob', 'clint'],
    tests_require=[],
    classifiers=[],
    test_suite='',
    entry_points={
        'console_scripts': [
            'azkm = azkm.__main__:main',
        ]
    },
    package_data={
        '': ['*.json', '*.tgz']
    },
    include_package_data=True
)



import json
import os
import random
import shutil

from azkm.utils import az
import azkm.utils.osutil as osutil
from azkm.providers.azurerm import (AzurermProvider, CognitiveAccount,
                                    KubernetesCluster, ResourceGroup,
                                    SearchService, StorageAccount, StorageContainer, StorageShare)
from cdktf import App, TerraformOutput, TerraformStack, TerraformVariable
from constructs import Construct

AZKM_DIR = os.path.join(os.path.expanduser ('~'), '.azkm')

def get_out_dir(km_id: str):
    out_dir = os.path.join(AZKM_DIR, '{0}.out'.format(km_id))
    if not os.path.isdir(AZKM_DIR):
        os.mkdir(AZKM_DIR)
    if not os.path.isdir(out_dir):
        os.mkdir(out_dir)
    return out_dir

def _clean_name(name: str):
    illegal_char = ['_', '-', ' ']
    for c in illegal_char:
        name = name.replace(c, '')
    return name

def _get_envvars(km_id: str):
    out_dir = get_out_dir(km_id)
    vars_file = os.path.join(out_dir, 'cdk.tf.json')
    if not os.path.isfile(vars_file):
        return {
            'env_id': km_id,
            'env_suffix': str(random.randint(0, 999999))
        }
    else:
        with open(vars_file, 'r') as f:
            vars = json.loads(f.read())['variable']
            
            def __get_var(var: str):
                candidates = [vars[k]['default'] for k in vars.keys() if var in k]
                if len(candidates):
                    return candidates[0]
                else:
                    return ''

            return {
                'env_id': __get_var('envid'),
                'env_suffix': __get_var('suffix'),
                'sp_app_id': __get_var('spappid'),
                'sp_obj_id': __get_var('spobjid'),
                'sp_secret': __get_var('spsecret')
            }

def _create_sp(km_id: str, suffix: str):
    assert az.logged_in(), 'please log into az cli with desired tenant, subscription and identity.'
    return az.create_sp('{0}{1}'.format(km_id, suffix))

class KmStack(TerraformStack):

    vars: dict = {}
    tfvars: dict = {}
    resources: dict = {}

    def __init__(self, scope: Construct, ns: str):
        super().__init__(scope, ns)

    def generate_vars(self, km_id: str, region: str):
        self.vars = _get_envvars(km_id)
        self.vars['env_id'] = km_id
        self.vars['env_region'] = region

        if 'sp_secret' not in self.vars:
            sp = _create_sp(self.vars['env_id'], self.vars['env_suffix'])
            self.vars['sp_app_id'] = sp['appId']
            self.vars['sp_obj_id'] = sp['objectId']
            self.vars['sp_secret'] = sp['password']

        self.tfvars['env_id'] = TerraformVariable(self, 'env_id', type='string', default=_clean_name(self.vars['env_id']))
        self.tfvars['env_region'] = TerraformVariable(self, 'env_region', type='string', default=self.vars['env_region'])
        self.tfvars['env_suffix'] = TerraformVariable(self, 'env_suffix', type='string', default=_clean_name(self.vars['env_suffix']))
        self.tfvars['sp_app_id'] = TerraformVariable(self, 'sp_app_id', type='string', default=self.vars['sp_app_id'])
        self.tfvars['sp_obj_id'] = TerraformVariable(self, 'sp_obj_id', type='string', default=self.vars['sp_obj_id'])
        self.tfvars['sp_secret'] = TerraformVariable(self, 'sp_secret', type='string', default=self.vars['sp_secret'])


    def generate_baseline(self, tags: dict):
    
        def _name_resource(res: str):
            return '{0}{1}{2}'.format(self.tfvars['env_id'].string_value, _clean_name(res), self.tfvars['env_suffix'].string_value)


        AzurermProvider(self, "Azurerm",
            features=[{}]
            )

        self.resources['rg'] = ResourceGroup(self, _name_resource('rg'),
            name=_name_resource('rg'), 
            location = self.vars['env_region'],
            tags = tags
            )

        self.resources['storage'] = StorageAccount(self, _name_resource('storage'),
            name=_name_resource('storage'),
            depends_on=[self.resources['rg']],
            resource_group_name=self.resources['rg'].name,
            location=self.resources['rg'].location, 
            account_tier='Standard',
            account_replication_type='GRS',
            tags=tags)

        self.resources['container_imagenet'] = StorageContainer(self, 'imagenet',
            name='imagenet',
            depends_on=[self.resources['storage']],
            storage_account_name  = self.resources['storage'].name,
            container_access_type = "private")

        self.resources['share_kmapp'] = StorageShare(self, 'kmapp',
            name='kmapp',
            depends_on=[self.resources['storage']],
            storage_account_name  = self.resources['storage'].name,
        )

        self.resources['cogsvcs'] = CognitiveAccount(self, _name_resource('cogsvcs'), 
            name=_name_resource('cogsvcs'),
            depends_on=[self.resources['rg']],
            resource_group_name=self.resources['rg'].name,
            location=self.resources['rg'].location, 
            sku_name='S0',
            kind = 'CognitiveServices'
            )

        self.resources['search'] = SearchService(self, _name_resource('search'), 
            name=_name_resource('search'),
            depends_on=[self.resources['rg']],
            resource_group_name=self.resources['rg'].name,
            location=self.resources['rg'].location, 
            sku='standard',
            timeouts={
                'create': '5m'
            }
            )

        self.resources['aks'] = KubernetesCluster(self, _name_resource('aks'), 
            name=_name_resource('aks'),
            depends_on=[self.resources['rg']],
            resource_group_name=self.resources['rg'].name,
            location=self.resources['rg'].location, 
            default_node_pool=[{
                'name': 'default',
                'nodeCount': 1,
                'vmSize': 'Standard_D4s_v3',
            }],
            dns_prefix='azkm',
            service_principal=[{
                'clientId': self.vars['sp_app_id'],
                'clientSecret': self.vars['sp_secret']
            }],
            addon_profile=[{
                'httpApplicationRouting': [{
                    'enabled': True
                }]
            }]
        )

        return self.resources

    def generate_outputs(self):
        return [TerraformOutput(self, r, value=self.resources[r].id) for r in self.resources]

def synth_km(km_id: str, region: str):
    app = App(outdir=get_out_dir(km_id))
    km_stack = KmStack(app, km_id)
    km_stack.generate_vars(km_id, region)
    km_stack.generate_baseline({'azkmid': km_id})
    km_stack.generate_outputs()
    app.synth()
    return app.outdir

def init(km_id: str):
    out_dir = get_out_dir(km_id)
    osutil.chdir(out_dir)
    osutil.run_subprocess(['terraform', 'init', '--upgrade'])

def plan(km_id: str):
    out_dir = get_out_dir(km_id)
    osutil.chdir(out_dir)
    osutil.run_subprocess(['terraform', 'plan'])

def apply(km_id: str):
    out_dir = get_out_dir(km_id)
    osutil.chdir(out_dir)
    osutil.run_subprocess(['terraform', 'apply'])

def destroy(km_id: str):
    out_dir = get_out_dir(km_id)
    vars = _get_envvars(km_id)
    osutil.chdir(out_dir)
    osutil.run_subprocess(['terraform', 'destroy'])
    if 'sp_obj_id' in vars and vars['sp_obj_id']:
        az.delete_sp(vars['sp_obj_id'])
    shutil.rmtree(out_dir)

def refresh(km_id: str):
    out_dir = get_out_dir(km_id)
    osutil.chdir(out_dir)
    osutil.run_subprocess(['terraform', 'refresh'])

def get_state(km_id: str):
    refresh(km_id)
    out_dir = get_out_dir(km_id)
    out_state = {}
    with open(os.path.join(out_dir,'terraform.tfstate'), 'r') as f:
        tfstate = json.loads(f.read())
        for r in tfstate['resources']:
            __ = out_state[r['type']] if r['type'] in out_state else []
            out_state[r['type']] = __ + [i['attributes'] for i in r['instances']]
        return out_state

def get_envs():
    if os.path.isdir(AZKM_DIR):
        return [d.replace('.out', '') for d in os.listdir(AZKM_DIR)]
    else:
        return None



import subprocess
import os
import azkm

ROOT_DIR = '/'.join(azkm.__file__.split('/')[:-2])

def chdir(rel_dir: str = ''):    
    os.chdir(os.path.join(ROOT_DIR, rel_dir))

def run_subprocess(cmd: list):
    subprocess.run(cmd)


import firehelper
from azkm.utils import tf
from azkm.utils import text
from tabulate import tabulate

def read(km_id: str):
    state = text.flatten_dict(tf.get_state(km_id))
    state_tabular = [['resource', 'value']]
    for k in state:
        state_tabular = state_tabular + [[k, state[k]]]
    print(tabulate(state_tabular, headers='firstrow'))

def list_env():
    print(tabulate([[env] for env in tf.get_envs()]))

env_cmd = {
    'environment': {
        'show': read,
        'list': list_env
    }
}

firehelper.CommandRegistry.register(env_cmd)


"""CLI commands: fulmar deploy dataset [imagenet, cord19]."""
import io
import os
import random
import tarfile

import azure.core.exceptions
import requests
from azure.storage.blob import BlobClient
from clint.textui import progress
from firehelper import CommandRegistry

from azkm.utils import tf
from azkm.utils.stdout import stdout_print

CORD19_URL = 'https://ai2-semanticscholar-cord-19.s3-us-west-2.amazonaws.com/historical_releases/cord-19_2020-10-12.tar.gz'
OBL_URL = 'https://abbottabad-compound-material.s3.amazonaws.com/Everything.20171105.tgz'
ENRON_URL = 'https://www.cs.cmu.edu/~./enron/enron_mail_20150507.tar.gz'


def deploy_imagenet(km_id: str, synset_id: str = None, num_images: int = 1000):
    """Deploy imagenet sample to a container.

    Args:
        km_id (str): azkm environment id
        synset_id (str, optional): imagenet synset id. Defaults to None.
        num_images (int, optional): number of images to download. Defaults to 1000.
    """
    try:
        env_state = tf.get_state(km_id)
        storage_conn = env_state['azurerm_storage_account'][0]['primary_blob_connection_string']
    except Exception as e:
        raise Exception('Error finding storage for environment {0}.'.format(km_id))
    img_count = 0
    while img_count < num_images:
        if not synset_id:
            # pick random synset
            syn_res = requests.get('http://www.image-net.org/api/text/imagenet.synset.obtain_synset_list')
            syn_res.raise_for_status()
            synset = syn_res.text.split('\n')
            synset_id = random.choice(synset)

        image_urls_res = requests.get(f'http://www.image-net.org/api/text/imagenet.synset.geturls?wnid={synset_id}')
        image_urls_res.raise_for_status
        images_to_fetch = image_urls_res.text.split('\r\n')[:-1][:num_images - img_count]
        print(f'fetching {len(images_to_fetch)} images from synset {synset_id}')
        for img_url in images_to_fetch:
            with io.BytesIO() as b_io:
                try:
                    counter = f'{img_count}/{num_images}'
                    spaces = 16 - len(counter)
                    stdout_print(f'{img_count}/{num_images}{"".join([" " for i in range(spaces)])}{img_url[:50]}')
                    res = requests.get(img_url, timeout=10)
                    if res.headers['Content-Type'][:5] == 'image':
                        b_io.write(res.content)
                        b_io.seek(0)
                        blob_name = f'imagenet/{synset_id}/{img_url.split("/")[-1]}'
                        with BlobClient.from_connection_string(conn_str=storage_conn, container_name='imagenet', blob_name=blob_name) as blob:
                            try:
                                blob.upload_blob(b_io)
                            except azure.core.exceptions.ResourceExistsError:
                                pass
                        img_count = img_count + 1
                    else:
                        stdout_print(f'not image\t{img_url[:50]}')
                except Exception as e:
                    stdout_print(f'error\t\t{img_url[:50]}')
                    pass
        synset_id = None
    print(f'{num_images} images fetched.')


dataset_commands = {
    "dataset": {
        "imagenet": deploy_imagenet
    }
}

CommandRegistry.register(dataset_commands)
